%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Copyright (c) Masahiro Sakai, 2001.
%
% 	Masahiro Sakai                  <s01397ms@sfc.keio.ac.jp>
%  
%  Redistribution and use in source and binary forms, with or without
%  modification, are permitted provided that the following conditions
%  are met:
%  
%  1. Redistributions of source code must retain the above copyright
%     notice, this list of conditions and the following disclaimer as
%     the first lines of this file unmodified.
%  2. Redistributions in binary form must reproduce the above copyright
%     notice, this list of conditions and the following disclaimer in the
%     documentation and/or other materials provided with the distribution.
%  
%  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
%  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
%  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
%  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
%  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
%  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
%  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
%  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
%  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 󥿡ե

ucs_utf8(UCS, UTF8) :- ucs_utf8(UCS, UTF8, []).

% ƥȥ
% file_get('example/utf8_text.txt', X), ucs_utf8(UCS, X), ucs_utf8(UCS, X2), file_put('example/utf8_text3.txt', X2).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ʸѴ

ucs_utf8([]) --> [].
ucs_utf8([H|T]) --> ucs_utf8mb(H), ucs_utf8(T).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ʸñ̤Ѵ

% UTF-8 -> UCS Ѵ
ucs_utf8mb(UCS) --> [X],
	{ atomic(X), !,
	  utf8_byte_type(X, LEN, BASE) },
	utf8mb_tail_bytes(Y, LEN),
	{ UCS is ((X - BASE) << (6 * LEN)) \/ Y }.

% UCS -> UTF-8 Ѵ
% FIXME: ƵȤäƥץ

ucs_utf8mb(UCS) -->
	{ UCS =< 0x0000007F }, !,
	[UCS].

ucs_utf8mb(UCS) -->
	{ UCS =< 0x000007FF, !,
	  X1 is (UCS >> 6) \/ 0xC0,
	  X2 is (UCS /\ 0x3F) \/ 0x80
	}, [X1, X2].

ucs_utf8mb(UCS) -->
	{ UCS =< 0x0000FFFF, !,
	  X1 is ((UCS >> 12) \/ 0xE0),
	  X2 is ((UCS >>  6) /\ 0x3F) \/ 0x80,
	  X3 is ((UCS >>  0) /\ 0x3F) \/ 0x80
        }, [X1, X2, X3].

ucs_utf8mb(UCS) -->
	{ UCS =< 0x001FFFFF, !,
	  X1 is ((UCS >> 18) \/ 0xF0),
	  X2 is ((UCS >> 12) /\ 0x3F) \/ 0x80,
	  X3 is ((UCS >>  6) /\ 0x3F) \/ 0x80,
	  X4 is ((UCS >>  0) /\ 0x3F) \/ 0x80
	}, [X1, X2, X3, X4].

ucs_utf8mb(UCS) -->
	{ UCS =< 0x03FFFFFF, !,
	  X1 is ((UCS >> 24) \/ 0xF8),
	  X2 is ((UCS >> 18) /\ 0x3F) \/ 0x80,
	  X3 is ((UCS >> 12) /\ 0x3F) \/ 0x80,
	  X4 is ((UCS >>  6) /\ 0x3F) \/ 0x80,
	  X5 is ((UCS >>  0) /\ 0x3F) \/ 0x80
        }, [X1, X2, X3, X4, X5].

ucs_utf8mb(UCS) -->
	{ UCS =< 0x7FFFFFFF, !,
	  X1 is ((UCS >> 30) \/ 0xFC),
	  X2 is ((UCS >> 18) /\ 0x3F) \/ 0x80,
	  X3 is ((UCS >> 18) /\ 0x3F) \/ 0x80,
	  X4 is ((UCS >> 12) /\ 0x3F) \/ 0x80,
	  X5 is ((UCS >>  6) /\ 0x3F) \/ 0x80,
	  X6 is ((UCS >>  0) /\ 0x3F) \/ 0x80
        }, [X1, X2, X3, X4, X5, X6].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% UTF-8 -> UCS 롼
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

utf8mb_tail_byte(Y) --> [X],
	{ utf8_byte_type(X, -1, BASE), Y is X - BASE }.
utf8mb_tail_bytes(0, 0) --> [], !.
utf8mb_tail_bytes(X, N) -->
	{ N1 is N - 1 },
	utf8mb_tail_byte(X1),
	utf8mb_tail_bytes(X2, N1),
	{ X is (X1 << (6 * N1)) \/ X2 }.

% utf8_byte_type(X, TAIL_LEN, BASE)
utf8_byte_type(X, 0,  BASE) :- BASE = 0   , X >= BASE, X =< 0x7F. % X: 0xxxxxxx
utf8_byte_type(X, 1,  BASE) :- BASE = 0xC0, X >= BASE, X =< 0xDF. % X: 110xxxxx
utf8_byte_type(X, 2,  BASE) :- BASE = 0xE0, X >= BASE, X =< 0xEF. % X: 1110xxxx
utf8_byte_type(X, 3,  BASE) :- BASE = 0xF0, X >= BASE, X =< 0xF7. % X: 11110xxx
utf8_byte_type(X, 4,  BASE) :- BASE = 0xF8, X >= BASE, X =< 0xFB. % X: 111110xx
utf8_byte_type(X, 5,  BASE) :- BASE = 0xFC, X >= BASE, X =< 0xFD. % X: 1111110x
utf8_byte_type(X, -1, BASE) :- BASE = 0x80, X >= BASE, X =< 0xBF. % X: 10xxxxxx
