%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Copyright (c) Masahiro Sakai, 2001.
%
%       Masahiro Sakai                  <s01397ms@sfc.keio.ac.jp>
%  
%  Redistribution and use in source and binary forms, with or without
%  modification, are permitted provided that the following conditions
%  are met:
%  
%  1. Redistributions of source code must retain the above copyright
%     notice, this list of conditions and the following disclaimer as
%     the first lines of this file unmodified.
%  2. Redistributions in binary form must reproduce the above copyright
%     notice, this list of conditions and the following disclaimer in the
%     documentation and/or other materials provided with the distribution.
%  
%  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
%  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
%  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
%  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
%  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
%  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
%  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
%  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
%  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- ['xml-dcg'].

% $B%I%-%e%a%s%H$N=hM}(B

xbel_to_xhtml(XBEL, XHTML) :-
        XBEL  = xml_document(_, ROOT, _),
        XHTML = xml_document(PROLOG, H_ROOT, []),

        xbel_to_xhtml_items(ROOT, [H_ROOT]),

        DOCTYPE = [ "html",
                    xml_ExternalID(
                        "-//W3C//DTD XHTML 1.0 Transitional//EN",
                        "http://www.w3.org/TR/xhtml1/DTD/loose.dtd"),
                    [] ],
        PROLOG  = xml_prolog(["1.0", "utf-8", ""],
                             "",
                             DOCTYPE,
                             "").


% $B%k!<%H%(%l%a%s%H$N=hM}(B

xbel_to_xhtml_items(XBEL, [XHTML]) :-
        XBEL  = xml_element("xbel", [], XBEL_CONTENT),
        XHTML = xml_element("html",
                            [ ["xmlns",    "http://www.w3.org/1999/xhtml"],
                              ["lang",     "ja"],
                              ["xml:lang", "ja"] ],
                            [HEAD, BODY] ),

         (member(xml_element("title", _, Title), XBEL_CONTENT); Title = [""]),
         (member(xml_element("description", _, [DESC]), XBEL_CONTENT); DESC = ""),
         
         HEAD = xml_element("head",
		  [],
                  [ xml_element("meta",
                    		 [ ["http-equiv", "Content-Type"],
                    		   ["content",    "text/html; charset=utf-8"] ],
                    		 []),
                    xml_element("meta",
                    		[ ["name",     "description"],
                    		  ["contents", DESC] ],
                    		[]),
                    xml_element("title", [], Title)
		  ]),

         BODY = xml_element("body", [],
                            [ xml_element("dl", [], DL_CONTENT) ]),

         xbel_to_xhtml_content(XBEL_CONTENT, DL_CONTENT).


% $B%V%C%/%^!<%/$N=hM}(B
% FIXME: id$B%"%H%j%S%e!<%H(B

xbel_to_xhtml_items(xml_element("bookmark", Attrs, C), [DT,DD]) :-
        (member(xml_element("title", _, Title), C); Title = ["-"]),
        (member(xml_element("description", _ , DESC), C); DESC = ["-"]),
        member(["href", URI], Attrs),
        % FIXME: $B%"%H%j%S%e!<%H$b$A$c$s$HJQ49$7$J$/$A$c$$$1$J$$(B
        DT = xml_element("dt", [],
                         [xml_element("a", [["href", URI]], Title)]),
        DD = xml_element("dd", [], DESC).


% $B%U%)%k%@$N=hM}(B
% FIXME: id$B%"%H%j%S%e!<%H(B

xbel_to_xhtml_items(xml_element("folder", _, Content), R) :- !,
	(member(xml_element("title", _, Title), Content); Title = ["-"]),
	(member(xml_element("description", _, Desc), Content); Desc = ["-"]),
	DT  = xml_element("dt", [], Title),
	DD1 = xml_element("dd", [], Desc),
	DD2 = xml_element("dd", [], [ xml_element("dl", [], Content2) ]),
	xbel_to_xhtml_content(Content, Content2),
	R = [DT, DD1, DD2].

% $B$=$l0J30$O<N$F$k(B
xbel_to_xhtml_items(_, []).


% $B%j%9%H$rJQ49(B

xbel_to_xhtml_content([], []).
xbel_to_xhtml_content([H|T], C) :-
        xbel_to_xhtml_items(H, A),
        xbel_to_xhtml_content(T, B),
        append(A, B, C).
